# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('updater', '0004_auto_20150923_1230'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='status',
            options={'verbose_name_plural': 'Status', 'verbose_name': 'Status'},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import sys

from django.core.management.base import BaseCommand
from django.core.urlresolvers import reverse
from django.conf import settings as django_settings
from django.contrib.sites.shortcuts import get_current_site

from updater.models import Status
from updater.register import check_host, register_site


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument("--host", default=False, type=str)
        parser.add_argument('--noinput', default=False, dest="noinput", action='store_true',
                            help="Do NOT prompt the user for input of any kind.")

    def handle(self, *args, **options):

        host = options["host"] or get_current_site(None)

        if "django.contrib.admin" in django_settings.INSTALLED_APPS:
            self.stdout.write("It looks like you have Django's admin installed. This management command is intended to "
                              "be used as a fallback if the admin is not used. You can use Django "
                              "Updaters admin page at: {host}{admin_url}".format(
                host=host, admin_url=reverse("admin:updater_status_changelist")))

            if not options["noinput"]:
                if get_input("Abort: (y/n) ") in ["y" or "yes"]:
                    sys.exit(1)

        self.stdout.write("\nPlease note: The Django Updater service won't work on a dev environment.\n\n")
        status = Status.objects.get()

        url = check_host(host, token=status.site_token)

        if not url:
            self.stdout.write("\nUnable to find the host for this installation, tried {0}".format(host))
            self.stdout.write("This happens when you don't use Django's site framework, or you haven't set up "
                              "any sites yet. The host is usually your domain name, e.g 'example.com' or "
                              "'www.example.com'.")
            if options["noinput"]:
                # return early if noinput is set
                sys.exit(1)

        while not url:
            try:
                host = get_input("Host: ")
            except KeyboardInterrupt:
                sys.exit(1)
            url = check_host(host=host, token=status.site_token)

        self.stdout.write("Contacting online service to register this site.")
        base_url = url.replace(status.site_token + "/", "")
        success, data = register_site(host, base_url)
        if not success:
            self.stderr.write(str(data))
            sys.exit(1)
        self.stdout.write(str(data))
        sys.exit(0)


def get_input(prompt):  # pragma: no cover
    # py2 and py3 compatible input prompt
    if sys.version_info[0] >= 3:
        return input(prompt)
    return raw_input(prompt)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import logging
import socket

from django.core.mail.message import EmailMultiAlternatives
from django.template.loader import render_to_string
from django import conf

from .conf import settings

logger = logging.getLogger(__name__)


def send_notification(result):
    """
    Sends a notification.
    :param result: Dictionary containing all updates and security issues
    :return: True if all notifcations have been sent successfully
    """
    # only mails are supported right now. This might change, so we go for the more generic `send_notification`
    # as method name, but use it as a proxy to send_mail
    return send_mail(result, mail_from=conf.settings.SERVER_EMAIL, mail_to=settings.UPDATER_EMAILS)


def send_mail(result, mail_from, mail_to, fail_sitently=False):
    """
    Sends a notification email.
    :param result: Dictionary containing all updates and security issues
    :return: :bool: True if mail has been send successfully
    """

    subject = "Important: Security updates on %s" % result["site"] if result["security"] \
        else "Updates available on %s" % result["site"]
    txt_message = render_to_string("summary.txt", result)
    html_message = render_to_string("summary.html", result)
    mail = EmailMultiAlternatives(
        subject, txt_message, mail_from, mail_to)
    mail.attach_alternative(html_message, 'text/html')
    return mail.send(fail_silently=fail_sitently)


def is_allowed_host(request, allowed_hosts=settings.UPDATER_ALLOWED_HOSTS):
    """
    Gets all associated IP addresses of `allowed_hosts` and checks if the requesting ip is one of them
    :param request: request object
    :param allowed_hosts: list of allowed hosts, `"*"` matches everything
    :return:
    """
    if "*" in allowed_hosts:
        return True
    ip = get_client_ip(request)

    for host in allowed_hosts:
        if ":" in host:
            host, port = host.split(":")
        else:
            port = None

        try:
            # getaddrinfo returns a list of 5 tuples in which we are only interested in the first tuple of the
            # fourth tuple
            if ip in [resp[4][0] for resp in socket.getaddrinfo(host, port)]:
                return True
        except socket.gaierror as e:
            logger.warning("Unable to get IP address for hostname {}".format(host))

    return False


def is_allowed_ip(request, allowed_ips=settings.UPDATER_ALLOWED_IPS):
    return "*" in allowed_ips or get_client_ip(request) in allowed_ips


def get_client_ip(request):
    """
    http://stackoverflow.com/questions/4581789/how-do-i-get-user-ip-address-in-django
    :param request:
    """
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import requests
from requests.exceptions import ConnectionError, RequestException
from django.core.urlresolvers import reverse

from updater.conf import settings
from updater.models import Status


def check_host(host, token, proto=None):
    base_url = "{host}{url}".format(host=host, url=reverse("updater_run", kwargs={"token": token}))
    http_url, https_url = "://".join(["http", base_url]), "://".join(["https", base_url])
    if proto is None:
        if is_reachable_url(https_url + "?health=1"):
            return https_url
        elif is_reachable_url(http_url + "?health=1"):
            return http_url
    else:
        return is_reachable_url(proto + base_url + "?health=1")
    return False


def register_site(host, base_url):
    status = Status.objects.get()

    data = {"name": host, "base_url": base_url}
    headers = {"Authorization": "Token " + settings.UPDATER_TOKEN}
    try:
        r = requests.post(settings.UPDATER_BASE_URL + "/api/v1/sites/", data=data, headers=headers)

        if r.status_code == 403:
            return False, "Invalid token."
        if r.status_code != 201:
            return False, r.content

        json = r.json()

        status.registered = True
        status.site_token = json["site_token"]
        status.save()
        return True, "This site is now registered at djangoupdater.com"
    except ConnectionError:
        return False, "Unable to connect to the online service. Please try again later."
    except (RequestException, ValueError) as e:
        return False, e


def get_site_status(updater_token, site_token):

    service_url = "/".join([settings.UPDATER_BASE_URL, "api/v1/sites", site_token, ""])
    headers = {"Authorization": "Token " + updater_token}
    try:
        r = requests.get(service_url, headers=headers)

        if r.status_code == 200:
            return True, r.json()
        elif r.status_code in [403, 404]:
            return False, r.status_code
    except ConnectionError:
        return False, "Unable to connect to the online service. Please try again later."
    except RequestException as e:
        return False, e


def is_reachable_url(url):
    try:
        r = requests.get(url=url, timeout=2.0)
        if r.status_code == 200:
            return True
    except RequestException as e:
        pass
    return False


try:  # pragma: nocover
    from celery import shared_task

    @shared_task()
    def run_check():
        from updater import package
        package.run_check()
except ImportError: # pragma: nocover
    pass


# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from uuid import uuid4


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Status'
        db.create_table(u'updater_status', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('registered', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('site_token', self.gf('django.db.models.fields.CharField')(default=uuid4, max_length=36)),
        ))
        db.send_create_signal(u'updater', ['Status'])

        # Adding model 'Notification'
        db.create_table(u'updater_notification', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('security_issue', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'updater', ['Notification'])


    def backwards(self, orm):
        # Deleting model 'Status'
        db.delete_table(u'updater_status')

        # Deleting model 'Notification'
        db.delete_table(u'updater_notification')


    models = {
        u'updater.notification': {
            'Meta': {'object_name': 'Notification'},
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'security_issue': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        u'updater.status': {
            'Meta': {'object_name': 'Status'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'registered': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'site_token': ('django.db.models.fields.CharField', [], {'default': "uuid4", 'max_length': '36'})
        }
    }

    complete_apps = ['updater']


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('updater', '0002_notification'),
    ]

    operations = [
        migrations.CreateModel(
            name='Status',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('registered', models.BooleanField(default=False)),
            ],
        ),
        migrations.AlterField(
            model_name='notification',
            name='created',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from .models import Status
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse
from .package import run_check
from .util import is_allowed_host, is_allowed_ip


def run_view(request, token):
    """
    This is the view that kicks off the process to check for updates.
    :param request:
    :param token:
    :return:
    """
    if not is_allowed_host(request) or not is_allowed_ip(request):
        raise PermissionDenied

    if not Status.objects.filter(site_token=token).exists():
        raise PermissionDenied

    if not request.GET.get("health", False):
        run_check(registered=request.GET.get("registered", False))
    return HttpResponse("ok")




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('updater', '0003_auto_20150923_0854'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Token',
        ),
        migrations.AddField(
            model_name='status',
            name='site_token',
            field=models.CharField(default=uuid.uuid4, max_length=36),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from django.apps import AppConfig


class DjangoUpdaterAppConfig(AppConfig):

    name = 'updater'
    verbose_name = "Django Updater"


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import uuid
import django.utils.timezone


class Migration(migrations.Migration):

    replaces = [('updater', '0001_initial'), ('updater', '0002_notification'), ('updater', '0003_auto_20150923_0854'), ('updater', '0004_auto_20150923_1230'), ('updater', '0005_auto_20151013_1152')]

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, verbose_name='ID', serialize=False, primary_key=True)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('security_issue', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Status',
            fields=[
                ('id', models.AutoField(auto_created=True, verbose_name='ID', serialize=False, primary_key=True)),
                ('registered', models.BooleanField(default=False)),
                ('site_token', models.CharField(default=uuid.uuid4, max_length=36)),
            ],
        ),
        migrations.AlterModelOptions(
            name='status',
            options={'verbose_name': 'Status', 'verbose_name_plural': 'Status'},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from django.db import models
from uuid import uuid4
from django.utils import timezone


class StatusManager(models.Manager):
    """
    Singleton Manager implementation
    """

    def get(self):
        return self.get_or_create(pk=1)[0]


class Status(models.Model):
    """
    Holds the settings/status of the current installation
    """

    registered = models.BooleanField(default=False)
    site_token = models.CharField(max_length=36, default=uuid4)
    objects = StatusManager()

    class Meta:
        verbose_name = "Status"
        verbose_name_plural = "Status"


class Notification(models.Model):

    created = models.DateTimeField(default=timezone.now)
    security_issue = models.BooleanField(default=False)


# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import json
from distutils.version import StrictVersion

from django.contrib import admin
from django.template.response import TemplateResponse
from django.core.urlresolvers import reverse, NoReverseMatch
from django.conf.urls import url
from django.contrib.admin.templatetags.admin_static import static
from django.conf import settings as django_settings
from django.http import HttpResponse
from django.template.loader import render_to_string
from django import get_version

from updater.models import Status
from updater.conf import settings
from updater.register import get_site_status, register_site
from updater.package import get_packages


class StatusAdmin(admin.ModelAdmin):
    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_urls(self):
        urls = super(StatusAdmin, self).get_urls()
        return [
            url(r'^$', self.admin_site.admin_view(self.status_view), name="updater_status_changelist"),
            url(r'^registration/$', self.admin_site.admin_view(self.registration_status_view),
                name="updater_status_registration"),
        ] + urls

    def status_view(self, request):
        request.current_app = self.admin_site.name

        # see: https://github.com/django/django/blob/stable/1.9.x/django/contrib/admin/options.py

        context = dict(
            opts=self.opts,
            token=settings.UPDATER_TOKEN,
            packages=json.dumps(get_packages()),
            update_url="/".join([settings.UPDATER_BASE_URL, "api/v1/packages/"]),
            registration_url=reverse("admin:updater_status_registration"),
            # reversing admin:index and admin:app_list in the template gives strange results on django 1.4 and 1.5,
            # doing it here as a workaround
            admin_index=reverse("admin:index"),
            admin_app_list=reverse("admin:app_list", kwargs={"app_label": self.opts.app_label})

        )

        version = get_version()

        # add the admin_site context for django >= 1.8
        if StrictVersion(version) >= StrictVersion("1.8"):
            context.update(self.admin_site.each_context(request))

        # jquery is bundled in /vendor/ on django 1.9 and yes.gif changed to yes.svg
        if StrictVersion(version) >= StrictVersion("1.9a1"):
            jquery = "admin/js/vendor/jquery/jquery"
        else:
            jquery = "admin/js/jquery"
        context["js"] = [
            static("{}.min.js".format(jquery)) if django_settings.DEBUG else static("{}.js".format(jquery)),
            static("admin/js/jquery.init.js"),
        ]

        return TemplateResponse(request, "admin/status.html", context)

    def registration_status_view(self, request):
        request.current_app = self.admin_site.name
        status = self.get_registration_status(request)
        if StrictVersion(get_version()) >= StrictVersion("1.9a1"):
            status["yes_img"] = static("admin/img/icon-yes.svg")
        else:
            status["yes_img"] = static("admin/img/icon-yes.gif")

        return HttpResponse(json.dumps({
            "errors": status["errors"],
            "content": render_to_string("admin/registration.html", status)
        }), content_type="application/json")

    def get_registration_status(self, request, retries=0, reg_msg=None):

        free_version_notice = "Please note: This page is intended to be used with the paid service at " \
                              "djangoupdater.com. If you are using the free version, check out the documentation at" \
                              "<a href='https://github.com/jayfk/django-updater'>github</a> to see how update checks" \
                              "can be scheduled on your side."

        if retries >= 3:
            #
            return {"errors": True, "status": reg_msg}

        try:
            # check that the view can be resolved, use a bogus uuid4
            reverse("updater_run", kwargs={"token": "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee"})
        except NoReverseMatch:
            # reverse_url not set
            return {"errors": True, "status": "<br/><br/>".join(
                ["Add <code>url(r'^updater/', include('updater.urls'))</code> to <code>urls.py</code> and try again.",
                 free_version_notice])}

        if not settings.UPDATER_TOKEN:
            # UPDATER_TOKEN is not set
            return {"errors": True, "status": "<br/><br/>".join([
                "Add <code>UPDATER_TOKEN</code> to your settings file and try again.",
                free_version_notice])}

        status = Status.objects.get()

        if not status.registered:
            # site is not registered, do that now
            return self.get_registration_status(request, retries=retries + 1, reg_msg=self.register(request))

        # site is registered, get the status
        success, data = get_site_status(settings.UPDATER_TOKEN, status.site_token)

        if not success and data == 404:
            # site does not exist, re-register
            return self.get_registration_status(request, retries=retries + 1, reg_msg=self.register(request))
        elif not success and data == 403:
            # UPDATER_TOKEN is not good
            return {"errors": True, "status": "Your token <code>{token}</code> is invalid.<br/>"
                                              "Please check you subscription status and make sure that the token "
                                              "matches exactly the one from your "
                                              "<a target='blank' href='https://djangoupdater.com/users/'>"
                                              "dashboard</a>.".format(token=settings.UPDATER_TOKEN)}
        elif not success:
            # totally failed, data is the request exception
            return {"errors": True, "status": data}

        # all good
        return {"errors": False, "status": data}

    def register(self, request):
        # the site is not yet registered, do that now
        proto = "http://" if request.is_secure else "https://"
        base_url = reverse("updater_run", kwargs={"token": "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee"}).replace(
            "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/", ""
        )
        base_url = "".join([proto, request.META["HTTP_HOST"], base_url])
        return register_site(host=request.META["HTTP_HOST"], base_url=base_url)[1]


admin.site.register(Status, StatusAdmin)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from django.conf import BaseSettings, settings as django_settings


class Settings(BaseSettings):

    UPDATER_ALLOWED_IPS = getattr(django_settings, "UPDATER_ALLOWED_IPS", ["*"])
    UPDATER_ALLOWED_HOSTS = getattr(django_settings, "UPDATER_ALLOWED_HOSTS", ["*"])
    UPDATER_DAYS_BETWEEN_NOTIFICATION = getattr(django_settings, "UPDATER_DAYS_BETWEEN_NOTIFICATION", 7)
    UPDATER_NOTIFY_ON_UPDATES = getattr(django_settings, "UPDATER_NOTIFY_ON_UPDATES", True)
    UPDATER_EMAILS = getattr(django_settings, "UPDATER_EMAILS", [mail for name, mail in django_settings.ADMINS])
    UPDATER_TOKEN = getattr(django_settings, "UPDATER_TOKEN", False)
    UPDATER_BASE_URL = getattr(django_settings, "UPDATER_BASE_URL", "https://djangoupdater.com")

settings = Settings()